__all__ = ["status"]


import json
import socket
import prettytable
import colorama
from colorama import Fore
from ._cache import read_daemon_cache


def colorify(string, text, color):
    colorama.init()  # for windows
    return string.replace(text, color + text + Fore.RESET)


def status():
    out = prettytable.PrettyTable()
    out.field_names = ["host", "port", "kind", "name", "status", "busy"]
    out.align = "l"
    for daemon in read_daemon_cache():
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((daemon.host, daemon.port))
            s.sendall(b'{"jsonrpc":"2.0", "method": "busy", "id":"status"}')
            ident = json.loads(s.recv(1024))
            out.add_row(
                [daemon.host, daemon.port, daemon.kind, daemon.name, "online", ident["result"]]
            )
        except Exception as e:
            out.add_row([daemon.host, daemon.port, daemon.kind, daemon.name, "offline", "?"])
    out = out.get_string()
    out = colorify(out, "online", Fore.GREEN)
    out = colorify(out, "offline", Fore.RED)
    out = colorify(out, "False", Fore.GREEN)
    out = colorify(out, "True", Fore.RED)
    print(out)



__all__ = ["enable", "disable", "start", "stop", "restart", "reload"]

import getpass
import pathlib
import subprocess
import sys
import tempfile

import appdirs

from ._cache import add_config, read_daemon_cache

service_template = """
[Unit]
Description=yaqd-{kind}

[Service]
Type=simple
User={user}
ExecStart={executable} --config={config_path}
ExecReload=/bin/kill -HUP $MAINPID

[Install]
WantedBy=multi-user.target
"""

if sys.platform.startswith("win32"):
    try:
        nssm_exe = (
            subprocess.run(["where", "nssm.exe"], capture_output=True, check=True)
            .stdout.decode()
            .strip()
        )
    except Exception:
        nssm_exe = pathlib.Path(__file__).parent / "bin" / "nssm.exe"
    nssm_exe = str(nssm_exe)


def enable(kind, password=None):
    user = getpass.getuser()
    known_daemons = read_daemon_cache()
    try:
        daemon_data = next(d for d in known_daemons if d.kind == kind)
        config_path = daemon_data.config_filepath
    except:
        config_path = (
            pathlib.Path(appdirs.user_config_dir("yaqd", "yaq")) / kind / "config.toml"
        )
        add_config(config_path)
    if sys.platform.startswith("win32"):
        if password is None:
            raise ValueError("Windows services require a password")
        executable = (
            subprocess.run(["where", f"yaqd-{kind}"], capture_output=True, check=True)
            .stdout.decode()
            .strip()
        )
        subprocess.run(
            [
                nssm_exe,
                "install",
                f"yaqd-{kind}",
                executable,
                f"--config={config_path}",
            ],
            check=True,
        )
        subprocess.run(
            [nssm_exe, "set", f"yaqd-{kind}", "ObjectName", f".\\{user}", password], check=True
        )

    elif sys.platform.startswith("linux"):
        executable = (
            subprocess.run(["which", f"yaqd-{kind}"], capture_output=True, check=True)
            .stdout.decode()
            .strip()
        )
        with tempfile.NamedTemporaryFile() as tf:
            tf.write(
                service_template.format(
                    kind=kind, executable=executable, config_path=config_path, user=user
                ).encode()
            )
            tf.flush()
            subprocess.run(
                ["sudo", "cp", tf.name, f"/etc/systemd/system/yaqd-{kind}.service"],
                check=True,
            )
            subprocess.run(
                ["sudo", "chmod", "+r", f"/etc/systemd/system/yaqd-{kind}.service"],
                check=True,
            )

        subprocess.run(["systemctl", "enable", f"yaqd-{kind}"])

    else:
        raise NotImplementedError


def disable(kind):
    if sys.platform.startswith("win32"):
        subprocess.run([nssm_exe, "remove", f"yaqd-{kind}"])
    elif sys.platform.startswith("linux"):
        subprocess.run(["systemctl", "disable", f"yaqd-{kind}"])
    else:
        raise NotImplementedError


def start(kind):
    if sys.platform.startswith("win32"):
        subprocess.run([nssm_exe, "start", f"yaqd-{kind}"])
    elif sys.platform.startswith("linux"):
        subprocess.run(["systemctl", "start", f"yaqd-{kind}"])
    else:
        raise NotImplementedError


def stop(kind):
    if sys.platform.startswith("win32"):
        subprocess.run([nssm_exe, "stop", f"yaqd-{kind}"])
    elif sys.platform.startswith("linux"):
        subprocess.run(["systemctl", "stop", f"yaqd-{kind}"])
    else:
        raise NotImplementedError


def restart(kind):
    if sys.platform.startswith("win32"):
        subprocess.run([nssm_exe, "restart", f"yaqd-{kind}"])
    elif sys.platform.startswith("linux"):
        subprocess.run(["systemctl", "restart", f"yaqd-{kind}"])
    else:
        raise NotImplementedError


def reload(kind):
    if sys.platform.startswith("win32"):
        subprocess.run([nssm_exe, "restart", f"yaqd-{kind}"])
    elif sys.platform.startswith("linux"):
        subprocess.run(["systemctl", "reload", f"yaqd-{kind}"])
    else:
        raise NotImplementedError



import argparse
import pathlib
import subprocess
import sys

import appdirs
import toml


def start(config_dir=None):
    if config_dir is None:
        config_dir = appdirs.user_config_dir("yaqd", "yaq")
    config = pathlib.Path(config_dir)
    tomls = config.rglob("*.toml")
    if not tomls:
        print(f"No config files found in {config}")
    for fp in config.rglob("*.toml"):
        if fp.stem.endswith("state"):
            continue
        with open(fp, "r") as f:
            try:
                cd = toml.load(f)
            except toml.TomlDecodeError as e:
                print(e, file=sys.stderr)
                print(fp, file=sys.stderr)
                continue
            if not cd.get("enable", True):
                continue
            if "entry" not in cd:
                continue

            cmd = [cd["entry"], "--config", str(fp)]
            print(fp)
            proc = subprocess.Popen(cmd, stdin=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
            print(f"PID: {proc.pid} - {' '.join(cmd)}")



__all__ = ["DaemonData"]


from dataclasses import asdict, dataclass, fields
from typing import Optional


@dataclass
class DaemonData:
    host: str
    port: int
    kind: str
    name: str
    config_filepath: str
    make: Optional[str] = None
    model: Optional[str] = None
    serial: Optional[str] = None

    def as_dict(self):
        return asdict(self)

    @classmethod
    def get_field_names(cls):
        return [f.name for f in fields(cls)]



__all__ = ["scan"]


import json
import socket
from pprint import pprint
from dataclasses import fields
from ._daemon_data import DaemonData
from ._cache import read_daemon_cache, write_to_daemon_cache


def scan(host="127.0.0.1", start=36000, stop=39999):
    # gather information about known daemons
    old = read_daemon_cache()
    old_ports = {}
    for dd in old:
        if dd.host == host:
            old_ports[dd.port] = dd
    #
    print(f"scanning host {host} from {start} to {stop}...")
    for i in range(start, stop + 1):
        # probe socket
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((host, i))
            s.sendall(b'{"jsonrpc":"2.0", "method": "id", "id":"find"}')
            ident = json.loads(s.recv(1024))
            kind = ident["result"]["kind"]
            name = ident["result"]["name"]
            s.sendall(b'{"jsonrpc":"2.0", "method": "config_filepath", "id":"find"}')
            config_filepath = json.loads(s.recv(1024))["result"]
        except Exception as e:
            if i in old_ports.keys():
                kind = old_ports[i].kind
                name = old_ports[i].name
                print(f"...known daemon {kind}:{name} on port {i} not responding")
            continue
        # format result
        kwargs = {k: v for k, v in ident["result"].items() if k in DaemonData.get_field_names()}
        kwargs["host"] = host
        kwargs["port"] = i
        kwargs["config_filepath"] = config_filepath
        dd = DaemonData(**kwargs)
        # give feedback
        if i not in old_ports.keys():
            print(f"...found new daemon {kind}:{name} on port {i}")
        elif old_ports[i] != dd:
            print(f"...updated daemon {kind}:{name} attributes on port {i}")
        else:
            print(f"...saw unchanged daemon {kind}:{name} on port {i}")
        # finish
        write_to_daemon_cache(dd)
        s.close()
    print("...done!")



__all__ = [list]


from ._cache import read_daemon_cache
import prettytable


def list():
    out = prettytable.PrettyTable()
    out.field_names = ["host", "port", "kind", "name"]
    out.align = "l"
    for daemon in read_daemon_cache():
        out.add_row([daemon.host, daemon.port, daemon.kind, daemon.name])
    print(out)



#! /usr/bin/env python3

import os
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))


def read(fname):
    return open(os.path.join(here, fname)).read()


with open(os.path.join(here, "yaqd_control", "VERSION")) as version_file:
    version = version_file.read().strip()

extra_files = {"yaqd_control": ["VERSION", "bin/nssm.exe"]}

setup(
    name="yaqd-control",
    packages=find_packages(exclude=("tests", "tests.*")),
    package_data=extra_files,
    python_requires=">=3.7",
    setup_requires=["pytest-runner"],
    tests_require=["pytest", "pytest-cov"],
    install_requires=["appdirs", "toml", "click", "prettytable", "colorama"],
    extras_require={
        "docs": ["sphinx", "sphinx-gallery>=0.3.0", "sphinx-rtd-theme"],
        "dev": ["black", "pre-commit", "pydocstyle"],
    },
    version=version,
    description="Core structures for yaq component daemons",
    # long_description=read("README.rst"),
    author="yaq Developers",
    author_email="git@ksunden.space",
    license="LGPL v3",
    url="https://yaq.fyi",
    project_urls={
        "Source": "https://gitlab.com/yaq/yaqd-control",
        "Documentation": "https://yaq.fyi",
        "Issue Tracker": "https://gitlab.com/yaq/yaqd-control/issues",
    },
    entry_points={"console_scripts": ["yaqd=yaqd_control.__main__:main"]},
    keywords="spectroscopy science multidimensional hardware",
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Scientific/Engineering",
    ],
)
